"""Constants for consistently converting between units.


FPS
fps(a) or ae
  (Absolute, or mass-based) Foot-Pound-Second System.

fps(g) or bg
  (Gravitational, or force-based) Foot-Pound-Second System. Also known as the
  technical FPS system or British Gravitational system

fps(e) or ee
  (weight-based) Foot-Pound-Second System. Also known as the English Engineering
  System.

cgs 
  Centimetre-Gram-Second System



See also:

- http://130.149.89.49:2080/v2016/books/popups/usb-tbl.html

"""

from .units import *
from . import fps, cgs, mks, iks, si, us
