
#
#  IKS 
#


#
#  Length 
#
mm           = millimeter   = 0.039370079         # Milimeter
cm           = centimeter   = 0.393700787         # Centimeter
m            = meter        = 39.37007874         # Meter
km           = kilometer    = 39370.07874         # Kilometer
inch                        = 1.0
ft           = foot         = 12.0                # International foot
yd           = yard         = 36.0                # International yard
mi           = mile         = 63360.0             # Mile

#
#  Angle 
#
rad          = radian       = 1.0                 # Radian
arcsec                      = 4.84814e-06         # Arcsecond
arcmin                      = 0.000290888         # Arcminute
deg          = degree       = 0.017453293         # Degrees

#
#  Constants 
#
pi           = pi           = 3.141592654         # Pi

#
#  Time 
#
sec          = second       = 1.0                 # second
min          = minute       = 60.0                # minute
hr           = hour         = 3600.0              # hour
day                         = 86400.0             # day
wk           = week         = 604800.0            # week
yr           = year         = 31536000.0          # year

#
#  Force 
#
N            = newton       = 0.000224809         # Newton (force)
dyn          = dyne         = 2.24809e-09         # Dyne
pdl          = poundal      = 3.10809e-05         # Poundal
lbf          = poundf       = 0.001
kip          = klbf         = 1.0

#
#  Mass 
#
slug                        = 8.33333e-05
lbm          = lbm          = 2.59008e-06         # International avoirdupois pound
gm           = gram         = 5.71015e-09
kg           = kilogram     = 5.71015e-06         # Kilogram
tonne                       = 0.005710147         # Metric tonne
oz           = ounce        = 1.6188e-07          # International avoirdupois ounce

#
#  Pressure 
#
Pa           = pascal       = 1.45038e-07         # Pascal
torr                        = 1.93368e-05
kPa          = kilopascal   = 0.000145038
bar                         = 0.014503774
atm          = atmosphere   = 0.014695949         # Standard atmosphere
MPa          = megapascal   = 0.145037736
GPa          = gigapascal   = 145.0377364
psi                         = 0.001               # Pound-square-inch
ksi                         = 1.0

#
#  Density
#
pcf                         = lbm / ft**3         # Pounds per cubic foot

#
#  Amount 
#
mol          = mole         = 1.0                 # Mole

#
#  Frequency 
#
Hz           = hertz        = 1.0                 

#
#  Area 
#
mm2                         = 0.001550003         # Square millimeter
m2                          = 0.15500031          # Square meter
cm2                         = 1550.0031           # Square centimeter
km2                         = 1550003100.0        # Square kilometer
in2          = inch2        = 1.0                 # Square inch
ft2          = feet2        = 144.0               # Square foot
yd2          = yard2        = 1296.0              # Square yard
mi2          = mile2        = 4014489600.0        # Square mile

#
#  Volume 
#
mm3                         = 6.10237e-05         
m3                          = 0.061023744         
cm3                         = 61023.74409         
km3                         = 61023700000000.0    
in3          = inch3        = 1.0                 
ft3          = foot3        = 1728.0              
cyd          = yard3        = 46656.0             
mi3          = mile3        = 254358000000000.0   

#
#  Velocity 
#
mmps                        = 0.039370079         # Millimeter per second
cps          = cmps         = 0.393700787         # Centimeter per second
mps                         = 39.37007874         # Meter per second
kps                         = 39370.07874         # Kilometer per second
ips          = inchps       = 1.0                 # Inch per second
fps          = footps       = 12.0                # Foot per second
yps                         = 36.0                # Yard per second
mph                         = 17.6                # mile per hour

#
#  Acceleration 
#
mmps2                       = 0.039370079
cps2         = cmps2        = 0.393700787
mps2                        = 39.37007874
kps2                        = 39370.07874
ips2         = inchps2      = 1.0
fps2         = footps2      = 12.0
yps2                        = 36.0
mph2                        = 0.004888889
gravity                     = 386.0885827         # Standard gravity

#
#  Angular Velocity 
#
rpm          = revpm        = 6.283185307         # Revolution per minute
radps                       = 1.0                 # Radian per second
