
#
#  MKS 
#

#  https://en.wikipedia.org/wiki/MKS_units



#
#  Length 
#
mm           = millimeter   = 0.001               # Milimeter
cm           = centimeter   = 0.01                # Centimeter
m            = meter        = 1.0                 # Meter
km           = kilometer    = 1000.0              # Kilometer
inch                        = 0.0254              
ft           = foot         = 0.3048              # International foot
yd           = yard         = 0.9144              # International yard
mi           = mile         = 1609.344            # Mile

#
#  Angle 
#
rad          = radian       = 1.0                 # Radian
arcsec                      = 4.84814e-06         # Arcsecond
arcmin                      = 0.000290888         # Arcminute
deg          = degree       = 0.017453293         # Degrees

#
#  Constants 
#
pi           = pi           = 3.141592654         # Pi

#
#  Time 
#
sec          = second       = 1.0                 # second
min          = minute       = 60.0                # minute
hr           = hour         = 3600.0              # hour
day                         = 86400.0             # day
wk           = week         = 604800.0            # week
yr           = year         = 31536000.0          # year

#
#  Force 
#
N            = newton       = 1.0                 # Newton (force)
dyn          = dyne         = 1e-05               # Dyne
pdl          = poundal      = 0.138254954         # Poundal
lbf          = poundf       = 4.448221657         
kip          = klbf         = 4448.221657         

#
#  Mass 
#
slug                        = 14.59390307
lbm          = lbm          = 0.453592368         # International avoirdupois pound
gm           = gram         = 0.001
kg           = kilogram     = 1.0                 # Kilogram
tonne                       = 1000.0              # Metric tonne
oz           = ounce        = 0.028349523         # International avoirdupois ounce

#
#  Pressure 
#
Pa           = pascal       = 1.0                 # Pascal
torr                        = 133.3223684         
kPa          = kilopascal   = 1000.0              
bar                         = 100000.0            
atm          = atmosphere   = 101325.0            # Standard atmosphere
MPa          = megapascal   = 1000000.0           
GPa          = gigapascal   = 1000000000.0        
psi                         = 6894.757358         # Pound-square-inch
ksi                         = 6894757.358         

#
#  Density 
#
pcf                         = lbm / ft**3         # Pounds per cubic foot

#
#  Amount 
#
mol          = mole         = 1.0                 # Mole

#
#  Frequency 
#
Hz           = hertz        = 1.0                 

#
#  Area 
#
mm2                         = 1e-06               # Square millimeter
m2                          = 0.0001              # Square meter
cm2                         = 1.0                 # Square centimeter
km2                         = 1000000.0           # Square kilometer
in2          = inch2        = 0.00064516          # Square inch
ft2          = feet2        = 0.09290304          # Square foot
yd2          = yard2        = 0.83612736          # Square yard
mi2          = mile2        = 2589988.11          # Square mile

#
#  Volume 
#
mm3                         = 1e-09               
m3                          = 1e-06               
cm3                         = 1.0                 
km3                         = 1000000000.0        
in3          = inch3        = 1.63871e-05         
ft3          = foot3        = 0.028316847         
cyd          = yard3        = 0.764554858         
mi3          = mile3        = 4168181825.0        

#
#  Velocity 
#
mmps                        = 0.001               # Millimeter per second
cps          = cmps         = 0.01                # Centimeter per second
mps                         = 1.0                 # Meter per second
kps                         = 1000.0              # Kilometer per second
ips          = inchps       = 0.0254              # Inch per second
fps          = footps       = 0.3048              # Foot per second
yps                         = 0.9144              # Yard per second
mph                         = 0.44704             # mile per hour

#
#  Acceleration 
#
mmps2                       = 0.001               
cps2         = cmps2        = 0.01                
mps2                        = 1.0                 
kps2                        = 1000.0              
ips2         = inchps2      = 0.0254              
fps2         = footps2      = 0.3048              
yps2                        = 0.9144              
mph2                        = 0.000124178         
gravity                     = 9.80665             # Standard gravity

#
#  Angular Velocity 
#
rpm          = revpm        = 6.283185307         # Revolution per minute
radps                       = 1.0                 # Radian per second
