
#
#  CGS 
#


#
#  Length 
#
mm           = millimeter   = 0.1                 # Milimeter
cm           = centimeter   = 1.0                 # Centimeter
m            = meter        = 100.0               # Meter
km           = kilometer    = 100000.0            # Kilometer
inch                        = 2.54                
ft           = foot         = 30.48               # International foot
yd           = yard         = 91.44               # International yard
mi           = mile         = 160934.4            # Mile

#
#  Angle 
#
rad          = radian       = 1.0                 # Radian
arcsec                      = 4.84814e-06         # Arcsecond
arcmin                      = 0.000290888         # Arcminute
deg          = degree       = 0.017453293         # Degrees

#
#  Constants 
#
pi           = pi           = 3.141592654         # Pi

#
#  Time 
#
sec          = second       = 1.0                 # second
min          = minute       = 60.0                # minute
hr           = hour         = 3600.0              # hour
day                         = 86400.0             # day
wk           = week         = 604800.0            # week
yr           = year         = 31536000.0          # year

#
#  Force 
#
N            = newton       = 100000.0            # Newton (force)
dyn          = dyne         = 1.0                 # Dyne
pdl          = poundal      = 13825.49538         # Poundal
lbf          = poundf       = 444822.1657         
kip          = klbf         = 444822165.7         

#
#  Mass 
#
slug                        = 14593.90307         
lbm          = lbm          = 453.592368          # International avoirdupois pound
gm           = gram         = 1.0                 
kg           = kilogram     = 1000.0              # Kilogram
tonne                       = 1000000.0           # Metric tonne
oz           = ounce        = 28.349523           # International avoirdupois ounce

#
#  Pressure 
#
Pa           = pascal       = 10.0                # Pascal
torr                        = 1333.223684         
kPa          = kilopascal   = 10000.0             
bar                         = 1000000.0
atm          = atmosphere   = 1013250.0           # Standard atmosphere
MPa          = megapascal   = 10000000.0          
GPa          = gigapascal   = 10000000000.0       
psi                         = 68947.57358         # Pound(f)-square-inch
ksi                         = 68947573.58         

#
#  Density 
#
pcf                         = lbm / ft**3         # Pounds per cubic foot

#
#  Amount 
#
mol          = mole         = 1.0                 # Mole

#
#  Frequency 
#
Hz           = hertz        = 1.0                 

#
#  Area 
#
mm2                         = 0.01                # Square millimeter
m2                          = 1.0                 # Square meter
cm2                         = 10000.0             # Square centimeter
km2                         = 10000000000.0       # Square kilometer
in2          = inch2        = 6.4516              # Square inch
ft2          = feet2        = 929.0304            # Square foot
yd2          = yard2        = 8361.2736           # Square yard
mi2          = mile2        = 25899881103.0       # Square mile

#
#  Volume 
#
mm3                         = 0.001               
m3                          = 1.0                 
cm3                         = 1000000.0           
km3                         = 1000000000000000.0  
in3          = inch3        = 16.387064           
ft3          = foot3        = 28316.84659         
cyd          = yard3        = 764554.858          
mi3          = mile3        = 4168180000000000.0  

#
#  Velocity 
#
mmps                        = 0.1                 # Millimeter per second
cps          = cmps         = 1.0                 # Centimeter per second
mps                         = 100.0               # Meter per second
kps                         = 100000.0            # Kilometer per second
ips          = inchps       = 2.54                # Inch per second
fps          = footps       = 30.48               # Foot per second
yps                         = 91.44               # Yard per second
mph                         = 44.704              # mile per hour

#
#  Acceleration 
#
mmps2                       = 0.1                 
cps2         = cmps2        = 1.0                 
mps2                        = 100.0               
kps2                        = 100000.0            
ips2         = inchps2      = 2.54                
fps2         = footps2      = 30.48               
yps2                        = 91.44               
mph2                        = 0.012417778         
gravity                     = 980.665             # Standard gravity

#
#  Angular Velocity 
#
rpm          = revpm        = 6.283185307         # Revolution per minute
radps                       = 1.0                 # Radian per second
