#
#  FPS 
#
Length = "foot"
Force  = "lbf"
Mass   = "slug"


#
#  Length 
#
mm           = millimeter   = 0.00328084          # Milimeter
cm           = centimeter   = 0.032808399         # Centimeter
m            = meter        = 3.280839895         # Meter
km           = kilometer    = 3280.839895         # Kilometer
inch                        = 0.083333333         
ft           = foot         = 1.0                 # International foot
yd           = yard         = 3.0                 # International yard
mi           = mile         = 5280.0              # Mile

#
#  Angle 
#
rad          = radian       = 1.0                 # Radian
arcsec                      = 4.84814e-06         # Arcsecond
arcmin                      = 0.000290888         # Arcminute
deg          = degree       = 0.017453293         # Degrees

#
#  Constants 
#
pi           = pi           = 3.141592654         # Pi

#
#  Time 
#
sec          = second       = 1.0                 # second
min          = minute       = 60.0                # minute
hr           = hour         = 3600.0              # hour
day                         = 86400.0             # day
wk           = week         = 604800.0            # week
yr           = year         = 31536000.0          # year

#
#  Force 
#
N            = newton       = 0.224808941         # Newton (force)
dyn          = dyne         = 2.24809e-06         # Dyne
pdl          = poundal      = 0.03108095          # Poundal
lbf          = poundf       = 1.0                 
kip          = klbf         = 1000.0              

#
#  Mass 
#
slug                        = 1.0                 
lbm          = lbm          = 0.03108095          # International avoirdupois pound
gm           = gram         = 6.85218e-05         
kg           = kilogram     = 0.068521765         # Kilogram
tonne                       = 68.52176521         # Metric tonne
oz           = ounce        = 0.001942559         # International avoirdupois ounce

#
#  Pressure 
#
Pa           = pascal       = 0.020885434         # Pascal
torr                        = 2.784495531         
kPa          = kilopascal   = 20.88543404         
bar                         = 2088.543404         
atm          = atmosphere   = 2116.216604         # Standard atmosphere
MPa          = megapascal   = 20885.43404         
GPa          = gigapascal   = 20885434.04         
psi                         = 144.0               # Pound-square-inch
ksi                         = 144000.0

#
#  Density 
#
pcf                         = lbm / ft**3         # Pounds per cubic foot

#
#  Amount 
#
mol          = mole         = 1.0                 # Mole

#
#  Frequency 
#
Hz           = hertz        = 1.0                 

#
#  Area 
#
mm2                         = 1.07639e-05         # Square millimeter
m2                          = 0.001076391         # Square meter
cm2                         = 10.76391042         # Square centimeter
km2                         = 10763910.42         # Square kilometer
in2          = inch2        = 0.006944444         # Square inch
ft2          = feet2        = 1.0                 # Square foot
yd2          = yard2        = 9.0                 # Square yard
mi2          = mile2        = 27878400.0          # Square mile

#
#  Volume 
#
mm3                         = 3.53147e-08         
m3                          = 3.53147e-05         
cm3                         = 35.31466672         
km3                         = 35314666721.0       
in3          = inch3        = 0.000578704         
ft3          = foot3        = 1.0                 
cyd          = yard3        = 27.0                
mi3          = mile3        = 147198000000.0      

#
#  Velocity 
#
mmps                        = 0.00328084          # Millimeter per second
cps          = cmps         = 0.032808399         # Centimeter per second
mps                         = 3.280839895         # Meter per second
kps                         = 3280.839895         # Kilometer per second
ips          = inchps       = 0.083333333         # Inch per second
fps          = footps       = 1.0                 # Foot per second
yps                         = 3.0                 # Yard per second
mph                         = 1.466666667         # mile per hour

#
#  Acceleration 
#
mmps2                       = 0.00328084          
cps2         = cmps2        = 0.032808399         
mps2                        = 3.280839895         
kps2                        = 3280.839895         
ips2         = inchps2      = 0.083333333         
fps2         = footps2      = 1.0                 
yps2                        = 3.0                 
mph2                        = 0.000407407         
gravity                     = 32.17404856         # Standard gravity

#
#  Angular Velocity 
#
rpm          = revpm        = 6.283185307         # Revolution per minute
radps                       = 1.0                 # Radian per second
